==> rechn_mgm_user_area-0.1.3.gem: app/models/devise_authentication/user_decorator.rb <==
# Decorator to add some special features to a User
# TODO: This is a strong relationship to DeviseAuthentication and will to stop work if we use anything different!! (se)
# TODO: Think whether move the implementation specific part (roles, favorites) to rechn_mgm_web? (se)


DeviseAuthentication::User.class_eval do
    # Hack to map module name. Otherwise has_many :organisations will fail..
    Organisation = RechnMgmUserArea::Organisation

    #.             1     2           4
    ROLES     = %w[admin buchhaltung sachbearbeiter]

    field :forename,        :type => String, :required => true
    field :lastname,        :type => String, :required => true
    field :company,         :type => String
    field :street,          :type => String
    field :street_no,       :type => String
    field :zipcode,         :type => String
    field :city,            :type => String
    field :return_address,  :type => String
    ## User Role
    field :favorites,       :type => Array,     :default => []

    attr_accessor :current_membership_id

    has_many :member_ships,  :class_name => RechnMgmUserArea::MemberShip
    has_many :organisations, :through => :member_ships

    after_create   :account_created
    after_destroy  :account_removed

    def user_name
        "#{self.forename} #{self.lastname}"
    end

    def is?(role)
        roles.include?(role.to_s)
    end

    def roles=(roles)
        self.roles_mask = (roles & ROLES).map { |r| 2**ROLES.index(r) }.inject(0, :+)
    end

    def roles
        ROLES.reject do |r|
            ((roles_mask.to_i || 0) & 2**ROLES.index(r)).zero?
        end
    end

    def set_organisation(organisation)
        self.current_membership_id = RechnMgmUserArea::MemberShip.where(user_id: self.id, organisation_id: organisation.id).first.id
    end

    def my_organisation
        self.member_ships.where(is_primary:true).first.organisation
    end

    def user_ids_of_my_organisation
        self.users_of_my_organisation.map{|u| u.id}
    end

    def users_of_my_organisation
        my_organisation = self.my_organisation
        my_organisation.users
    end

    def delete_user_from_my_organisation(user_id)
        membership_to_remove = RechnMgmUserArea::MemberShip.where(user_id: user_id, organisation_id: self.my_organisation.id).first
        fail "I cannot remove myself" if membership_to_remove.is_primary
        membership_to_remove.destroy

        # TODO: Documents beloning to this organisation must be returned to origin
    end

    def delete_membership_to_organisation(organisation_id)
        RechnMgmUserArea::MemberShip.find(organisation_id).destroy

        # TODO: Documents beloning to this organisation must be returned to origin
    end

    def add_user_to_my_organisation(user_id)
        my_organisation = self.my_organisation
        fail "User was already added to my organisation" if RechnMgmUserArea::MemberShip.where(user_id: user_id, organisation_id: my_organisation.id).count >= 1
        fail "Cannot add myself to my organistion" if user_id == self.id
        new_membership = RechnMgmUserArea::MemberShip.new(user_id: user_id, organisation_id: my_organisation.id, is_primary: false)

        new_membership.save
    end

    def invitation_state
        if self.invitation_token.present?
            :pending
        else
            :finished
        end
    end

    def buchhaltung_user_ids
        init_current_membership

        membership = RechnMgmUserArea::MemberShip.find(self.current_membership_id)
        organisation = membership.organisation

        RechnMgmUserArea::MemberShip.where(organisation_id: organisation.id, :roles_mask.ge => 2).map { |m| {id:m.user_id, roles_mask: m.roles_mask} }.keep_if { |m| self.roles_mask_matches_role(m[:roles_mask], 2) }.map {|m| m[:id] }
    end

    def sachbearbeiter_user_ids
        init_current_membership

        membership = RechnMgmUserArea::MemberShip.find(self.current_membership_id)
        organisation = membership.organisation

        RechnMgmUserArea::MemberShip.where(organisation_id: organisation.id, :roles_mask.ge => 2).map { |m| {id:m.user_id, roles_mask: m.roles_mask} }.keep_if { |m| self.roles_mask_matches_role(m[:roles_mask], 4) }.map {|m| m[:id] }
    end

    def roles_mask
        init_current_membership

        current_membership = RechnMgmUserArea::MemberShip.find(self.current_membership_id)
        current_membership.roles_mask
    end

    def roles_mask=(roles_mask)
        init_current_membership

        current_membership = RechnMgmUserArea::MemberShip.find(self.current_membership_id)
        current_membership.roles_mask = roles_mask
        current_membership.save
    end

    def roles_mask_matches_role(roles_mask, role_to_match)
        return (roles_mask | role_to_match) == roles_mask
    end

    private
    def account_created
        # Every user has its own organisation.
        my_organisation = RechnMgmUserArea::Organisation.new(name: "#{self.forename} #{self.lastname}s Organisation")
        my_membership   = nil
        success = my_organisation.save

        # Add himmself to it.
        if success
            my_membership = RechnMgmUserArea::MemberShip.new(user_id: self.id, organisation_id: my_organisation.id, is_primary: true, roles_mask: 3)

            success = my_membership.save
        end

        # Rollback if something went wrong.
        unless success
            my_organisation.destroy
            my_membership.destroy
            Rails.logger.error("Unable to create organisation for user #{self.id}")
        end
    end

    def account_removed
        #fail "Not implemented"
        puts "WARNING!!! It is not implemented"
        # TODO: How to remove organisations if users are deleted?
    end

    def init_current_membership
        if self.current_membership_id.blank?
            # Default to own Organisation if nothing else was specified.
            current_membership = RechnMgmUserArea::MemberShip.where( is_primary:true ).first
            if current_membership.present?
                self.current_membership_id = current_membership.id
            else
                Rails.logger.error "Unable to find the current membership"
            end
        end
    end



end

==> rechn_mgm_user_area-0.1.4.gem: app/models/devise_authentication/user_decorator.rb <==
# Decorator to add some special features to a User
# TODO: This is a strong relationship to DeviseAuthentication and will to stop work if we use anything different!! (se)
# TODO: Think whether move the implementation specific part (roles, favorites) to rechn_mgm_web? (se)


DeviseAuthentication::User.class_eval do
    # Hack to map module name. Otherwise has_many :organisations will fail..
    Organisation = RechnMgmUserArea::Organisation

    #.             1     2           4
    ROLES     = %w[admin buchhaltung sachbearbeiter]

    field :forename,        :type => String, :required => true
    field :lastname,        :type => String, :required => true
    field :company,         :type => String
    field :street,          :type => String
    field :street_no,       :type => String
    field :zipcode,         :type => String
    field :city,            :type => String
    field :return_address,  :type => String
    ## User Role
    field :favorites,       :type => Array,     :default => []

    attr_accessor :current_membership_id

    has_many :member_ships,  :class_name => RechnMgmUserArea::MemberShip
    has_many :organisations, :through => :member_ships

    after_create   :account_created
    after_destroy  :account_removed

    def user_name
        "#{self.forename} #{self.lastname}"
    end

    def is?(role)
        roles.include?(role.to_s)
    end

    def roles=(roles)
        self.roles_mask = (roles & ROLES).map { |r| 2**ROLES.index(r) }.inject(0, :+)
    end

    def roles
        ROLES.reject do |r|
            ((roles_mask.to_i || 0) & 2**ROLES.index(r)).zero?
        end
    end

    def current_membership_id=(value)
        @current_membership_id = value
        @current_membership = RechnMgmUserArea::MemberShip.find( @current_membership_id )
    end

    def set_organisation(organisation)
        self.current_membership_id = RechnMgmUserArea::MemberShip.where(user_id: self.id, organisation_id: organisation.id).first.id
    end

    def my_organisation
        self.member_ships.where(user_id: self.id, is_primary:true).first.organisation
    end

    def user_ids_of_current_organisation
        self.users_of_current_organisation.map{|u| u.id}
    end

    def users_of_current_organisation
        init_current_membership

        RechnMgmUserArea::MemberShip.where(organisation_id: @current_membership.organisation.id).map do |m|
            u = DeviseAuthentication::User.find( m.user_id )
            membership_of_user = RechnMgmUserArea::MemberShip.where(organisation_id: @current_membership.organisation.id, user_id:u.id).first
            u.current_membership_id = membership_of_user.id
            u
        end
    end

    def delete_user_from_my_organisation(user_id)
        membership_to_remove = RechnMgmUserArea::MemberShip.where(user_id: user_id, organisation_id: self.my_organisation.id).first
        fail "I cannot remove myself" if membership_to_remove.is_primary
        membership_to_remove.destroy

        # TODO: Documents beloning to this organisation must be returned to origin
    end

    def delete_membership_to_organisation(organisation_id)
        RechnMgmUserArea::MemberShip.find(organisation_id).destroy

        # TODO: Documents beloning to this organisation must be returned to origin
    end

    def add_user_to_my_organisation(user_id)
        my_organisation = self.my_organisation
        fail "User was already added to my organisation" if RechnMgmUserArea::MemberShip.where(user_id: user_id, organisation_id: my_organisation.id).count >= 1
        fail "Cannot add myself to my organistion" if user_id == self.id
        new_membership = RechnMgmUserArea::MemberShip.new(user_id: user_id, organisation_id: my_organisation.id, is_primary: false)

        new_membership.save
    end

    def invitation_state
        if self.invitation_token.present?
            :pending
        else
            :finished
        end
    end

    def buchhaltung_user_ids
        init_current_membership

        RechnMgmUserArea::MemberShip.where(organisation_id: @current_membership.organisation.id, :roles_mask.ge => 2).map { |m| {id:m.user_id, roles_mask: m.roles_mask} }.keep_if { |m| self.roles_mask_matches_role(m[:roles_mask], 2) }.map {|m| m[:id] }
    end

    def sachbearbeiter_user_ids
        init_current_membership

        RechnMgmUserArea::MemberShip.where(organisation_id: @current_membership.organisation.id, :roles_mask.ge => 2).map { |m| {id:m.user_id, roles_mask: m.roles_mask} }.keep_if { |m| self.roles_mask_matches_role(m[:roles_mask], 4) }.map {|m| m[:id] }
    end

    def roles_mask
        init_current_membership

        @current_membership.roles_mask
    end

    def roles_mask=(roles_mask)
        init_current_membership

        @current_membership.roles_mask = roles_mask
        @current_membership.save
    end

    def roles_mask_matches_role(roles_mask, role_to_match)
        return (roles_mask | role_to_match) == roles_mask
    end

    private
    def account_created
        # Every user has its own organisation.
        my_organisation = RechnMgmUserArea::Organisation.new(name: "#{self.forename} #{self.lastname}s Organisation")
        my_membership   = nil
        success = my_organisation.save

        # Add himmself to it.
        if success
            my_membership = RechnMgmUserArea::MemberShip.new(user_id: self.id, organisation_id: my_organisation.id, is_primary: true, roles_mask: 3)

            success = my_membership.save
        end

        # Rollback if something went wrong.
        unless success
            my_organisation.destroy
            my_membership.destroy
            Rails.logger.error("Unable to create organisation for user #{self.id}")
        end
    end

    def account_removed
        #fail "Not implemented"
        puts "WARNING!!! It is not implemented"
        # TODO: How to remove organisations if users are deleted?
    end

    def init_current_membership
        if self.current_membership_id.blank?
            # Default to own Organisation if nothing else was specified.
            current_membership = RechnMgmUserArea::MemberShip.where( user_id: self.id, is_primary:true ).first
            if current_membership.present?
                self.current_membership_id = current_membership.id
            else
                Rails.logger.error "Unable to find the current membership. user: #{self.id}"
            end
        end
    end



end
